extends Node

@export var player_scene: PackedScene
@export var resource_scene: PackedScene

@export var max_resources = 150
@export var resource_chance = float( max_resources ) / ( float( max_resources ) * 2 )
var resource_spacing = 1

@onready var game_area = get_node( "NavigationRegion3D/GameArea" )

func _ready( ) :
	# Create the randomized Game Area
	for b in max_resources :
		#continue
		create_resource( float( b ), true )

	# Add the player
	add_player( )
	
func add_player( ) :
	#ui = player_ui.instantiate( )
	#ui.initialize( health, energy, experience )
	# using player_count here will be a problem, but works for now since we don't have multiplayer anyway
	#get_node( player_ui_positions[ player_count ] ).add_child( ui )
	
	var new_player = player_scene.instantiate( )
	var spawn_point = $NavigationRegion3D/GameArea/PlayerSpawnPoint
	new_player.initialize( spawn_point )
	game_area.add_child( new_player )

func create_resource( resource_num, game_start ) :
	# Create a new instance of the resource scene.
	# We may want to add something here for different resource types
	# like some sort of glass/crystal that cannot be destroyed by laser
	# weapons, instead it refracts the laser as it passes through and
	# shoot things on the other side.
	if get_tree( ) != null :
		var resouces = get_tree( ).get_nodes_in_group( "resource_source" )
		resource_chance = 1.0 - ( float( resouces.size( ) ) / float( max_resources ) )
	else :
		print( "No tree to find resouces" )
	
	if randf( ) < resource_chance :
		print( "Generating resource ", resource_num )
		var resource  = resource_scene.instantiate( )
		# Choose a random location on the SpawnPath.
		# We store the reference to the SpawnLocation node.
		var resource_spawn_location = get_node( "NavigationRegion3D/GameArea/ResourceSpawnPath/ResourceSpawnLocation" )
		resource_spawn_location.progress_ratio = float( resource_num ) / float( max_resources )
		resource.initialize( resource_spawn_location.position, game_start, resource_num )
		# Spawn the mob by adding it to the Main scene.
		game_area.add_child( resource )
		check_position( resource, resource_num )

func check_position( new_resource, resource_num ) :
	# Don't allow spawning inside another resouce
	if get_tree( ) != null :
		var resources = get_tree( ).get_nodes_in_group( "resource_source" )
		var r = 0
		for resource in resources :
			if r != resource_num :
				var resource_pos = new_resource.position.distance_to( resource.position )
				if resource_pos < resource_spacing :
					new_resource.position.x = new_resource.position.x + randf_range( resource_spacing * -1, resource_spacing )
					new_resource.position.z = new_resource.position.z + randf_range( resource_spacing * -1, resource_spacing )
			r += 1
	else :
		print( "There is no tree of resources?" )
